import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';

/// Abstract class Pagination Service
/// to provide a method to fetch data from external service
abstract class PaginationService {
  /// [getData] fetches function from external service to build the [itemList]
  /// Has an optional parameter [lastFetched] to indicate the last item fetched
  /// and construct next page from there.
  Future<List> getData([lastFetched]);
}

/// For loading provider
class Loading extends ChangeNotifier {
  static bool _loading = false;
  toggle() {
    _loading = !_loading;
    notifyListeners();
  }

  get getLoading => _loading;
}

final loadingProvider = ChangeNotifierProvider.autoDispose<Loading>((ref) {
  Loading loading = Loading();
  return loading;
});

/// Buttons
class NavigationButtons extends StatelessWidget {
  final Function nextPage;
  final Function previousPage;
  final Function goToPage;
  final bool navButtons;
  final int currentPage;
  final int numOfPages;
  final Color color;
  NavigationButtons({
    this.nextPage,
    this.previousPage,
    this.navButtons,
    this.goToPage,
    this.currentPage = 0,
    this.numOfPages,
    this.color,
  });

  List<Widget> buildButtons(BuildContext context) {
    List<Widget> navigationButtons = [];
    navigationButtons.add(const SizedBox(width: 10));
    navigationButtons.add(
      GestureDetector(
        onTap: () => goToPage(0),
        child: Container(
          width: 35,
          height: 35,
          padding: EdgeInsets.all(4),
          decoration: BoxDecoration(
            border: currentPage == 0
                ? Border.all(color: Theme.of(context).accentColor, width: 3)
                : null,
            color: color != null ? color : Theme.of(context).backgroundColor,
            borderRadius: BorderRadius.all(Radius.circular(8)),
          ),
          child: Center(child: Text('1', style: TextStyle(fontSize: 18))),
        ),
      ),
    );
    navigationButtons.add(const SizedBox(width: 10));
    currentPage > 0 && currentPage <= numOfPages
        ? navigationButtons.addAll([
            GestureDetector(
              onTap: () => goToPage(currentPage),
              child: Container(
                width: 35,
                height: 35,
                padding: EdgeInsets.all(4),
                decoration: BoxDecoration(
                    border: Border.all(
                        color: Theme.of(context).accentColor, width: 3),
                    color: color != null
                        ? color
                        : Theme.of(context).backgroundColor,
                    borderRadius: BorderRadius.all(Radius.circular(8))),
                child: Center(
                    child: Text('${currentPage + 1}',
                        style: TextStyle(fontSize: 18))),
              ),
            ),
            const SizedBox(width: 10),
          ])
        : navigationButtons.add(const SizedBox());

//    currentPage > 1 && currentPage != numOfPages - 1
//        ? navigationButtons.addAll([
//            GestureDetector(
//              onTap: () => goToPage(numOfPages - 1),
//              child: Container(
//                width: 35,
//                height: 35,
//                padding: EdgeInsets.all(4),
//                decoration: BoxDecoration(
//                    color: Theme.of(context).backgroundColor,
//                    borderRadius: BorderRadius.all(Radius.circular(8))),
//                child: Center(
//                    child:
//                        Text('${numOfPages}', style: TextStyle(fontSize: 18))),
//              ),
//            ),
//            const SizedBox(width: 10),
//          ])
//        : navigationButtons.add(const SizedBox());

    return navigationButtons;
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        GestureDetector(
          onTap: () => previousPage(),
          child: Container(
            width: 35,
            height: 35,
            padding: EdgeInsets.all(4),
            decoration: BoxDecoration(
                color:
                    color != null ? color : Theme.of(context).backgroundColor,
                borderRadius: BorderRadius.all(Radius.circular(8))),
            child: Icon(Icons.arrow_back_ios_rounded),
          ),
        ),
        navButtons == false
            ? const SizedBox(width: 10)
            : Row(children: buildButtons(context)),
        GestureDetector(
          onTap: () => nextPage(),
          child: Container(
            width: 35,
            height: 35,
            padding: EdgeInsets.all(4),
            decoration: BoxDecoration(
                color:
                    color != null ? color : Theme.of(context).backgroundColor,
                borderRadius: BorderRadius.all(Radius.circular(8))),
            child: Icon(Icons.arrow_forward_ios_rounded),
          ),
        ),
      ],
    );
  }
}

class AltScreen extends StatelessWidget {
  final Function refresh;
  AltScreen({Key key, this.errorText = 'Error Fetching', this.refresh})
      : super(key: key);
  final String errorText;
  @override
  Widget build(BuildContext context) {
    return Center(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Text(
            errorText,
            style:
                TextStyle(color: Theme.of(context).accentColor, fontSize: 28),
          ),
          const SizedBox(height: 5),
          GestureDetector(
            child: Container(
              child: Text('Try Again'),
              padding: EdgeInsets.all(8),
              decoration: BoxDecoration(
                  color: Theme.of(context).backgroundColor,
                  borderRadius: BorderRadius.all(Radius.circular(8))),
            ),
            onTap: () => refresh(),
          )
        ],
      ),
    );
  }
}
