import 'pagination_controller.dart';
import 'pagination_utils.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:flutter/foundation.dart' show kIsWeb;

class PagedListView extends StatelessWidget {
  final ItemBuilder
      itemBuilder; // itemBuilder is a typedef function that returns a Widget

  /// Accepts a PaginationService
  /// which provides a method to fetch data from external service
  final PaginationService service;

  /// Number of results to show per page
  final int resultsPerPage;

  /// If true show first and current page buttons
  final bool navButtons;

  /// Color for page buttons
  final Color buttonColor;

  PagedListView({
    @required this.itemBuilder,
    @required this.service,
    this.resultsPerPage = 10,
    this.navButtons = false,
    this.buttonColor,
  });

  @override
  Widget build(BuildContext context) {
    return ProviderScope(
      child: PaginatedListView(
          itemBuilder: itemBuilder,
          service: service,
          resultsPerPage: resultsPerPage,
          navButtons: navButtons,
          buttonColor: buttonColor),
    );
  }
}

typedef ItemBuilder = Widget Function(dynamic item, int index);

/// Creates a paginated ListView.
/// Parameter [service] accepts an object of type [PaginationService] which provides [getData]
/// function to fetch more data into the list. itemBuilder: Receives a function that returns a
/// widget to use with each item, similar to ListView.

class PaginatedListView extends ConsumerWidget {
  final ItemBuilder
      itemBuilder; // itemBuilder is a typedef function that returns a Widget
  final PaginationService service;
  final int resultsPerPage;
  final bool navButtons;
  final Color buttonColor;

  PaginatedListView({
    @required this.itemBuilder,
    @required this.service,
    this.resultsPerPage = 10,
    this.navButtons = false,
    this.buttonColor,
  });

  @override
  Widget build(BuildContext context, ScopedReader watch) {
    ControllerParameters parameters =
        ControllerParameters(service: service, resultsPerPage: resultsPerPage);

    final ScrollController scrollController = ScrollController();
    final paginationController =
        watch(paginationControllerProvider(parameters));
    final loading = watch(loadingProvider);

    /// For Mobile Platforms
    refresh() async {
      print('refresh');
      loading.toggle();
      print(loading.getLoading);
      await Future.delayed(const Duration(milliseconds: 500));
      paginationController.fetch();
      loading.toggle();
    }

    /// Add listener to scrollController and fetch more data if at the end
    scrollController.addListener(() {
      if (scrollController.offset >=
              scrollController.position.maxScrollExtent &&
          !scrollController.position.outOfRange) {
        refresh();
      }
    });

    /// For WEB Platforms
    List<Widget> buildItemList() {
      List<Widget> displayedItems = [];
      for (int i = 0; i < paginationController.displayItems().length; i++) {
        displayedItems.add(itemBuilder(paginationController.displayItems()[i],
            paginationController.pageIndex * resultsPerPage + i));
      }
      displayedItems.add(
        NavigationButtons(
          nextPage: paginationController.increasePageIndex,
          previousPage: paginationController.decreasePageIndex,
          navButtons: navButtons,
          color: buttonColor,
          numOfPages: paginationController.getSize ~/
              paginationController.resultsPerPage,
          currentPage: paginationController.pageIndex,
          goToPage: (int i) {
            paginationController.setPageIndex(i);
            print('pageIndex: ${paginationController.pageIndex}');
          },
        ),
      );
      return displayedItems;
    }

    return Material(
      child: Builder(builder: (context) {
        if (paginationController.refreshError)
          return AltScreen(
              errorText: paginationController.errorMessage,
              refresh: () => refresh());
        else if (paginationController.getList.isEmpty)
          return AltScreen(
              errorText: 'Sorry! Empty Page', refresh: () => refresh());
        else

          /// Change for refresh indicator if platform is mobile
          return Container(
            width: MediaQuery.of(context).size.height * 0.7,
            child:

                /// For Mobile Platforms
                !kIsWeb
                    ? RefreshIndicator(
                        onRefresh: () => refresh(),
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.center,
                          children: [
                            Expanded(
                              child: ListView.builder(
                                  controller: scrollController,
                                  shrinkWrap: true,
                                  // Make the scrollable widget always scrollable so [onRefresh] in RefreshIndicator is always enable
                                  physics: const BouncingScrollPhysics(
                                      parent: AlwaysScrollableScrollPhysics()),
                                  itemBuilder: (context, index) {
                                    /// itemBuilder is a function that returns a Widget.
                                    /// paginationController.getList[index] is the item data
                                    /// from the list to be displayed
                                    /// itemBuilder function wraps the item data on a widget
                                    return itemBuilder(
                                        paginationController.getList[index],
                                        index);
                                  },
                                  itemCount: paginationController.getSize),
                            ),
                            loading.getLoading
                                ? Center(child: CircularProgressIndicator())
                                : Container(),
                          ],
                        ),
                      )
                    : Column(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                          Expanded(
                            child: ListView(
                              shrinkWrap: true,
                              children: buildItemList(),
                            ),
                          ),
                        ],
                      ),
          );
      }),
    );
  }
}
